//#########################################################
//#
//# Titre : 	Utilitaires BD LINUX Automne 08
//#				SIF-1015 - Système d'exploitation
//#				Université du Québec à Trois-Rivières
//#
//# Auteur : 	Francois Meunier et David Laperriere
//#	Date :		Septembre 2008
//#
//# Langage : 	ANSI C on LINUX 
//#
//#######################################

#include "gestionListeChaineeBD.h"

//Pointeur de tête de liste
extern struct record* head;
//Pointeur de queue de liste pour ajout rapide
extern struct record* queue;

//#######################################
//#
//# Affiche une série de retour de ligne pour "nettoyer" la console
//#
void cls(void){
	printf("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");
	}

//#######################################
//#
//# Affiche un messgae et quitte le programme
//#
void error(const int exitcode, const char * message){
	printf("\n-------------------------\n%s\n",message);
	exit(exitcode);
	}

//#######################################
//#
//# Chargement de la BD binaire
//#
void loadDB(){
	//Initialisation des pointeurs
	head = NULL;
	queue = NULL;

	//Ajout de l'élément bidon
	addItem("", "", 0);

	//Ouverture du fichier source en mode "rb" : [r]ead [b]inary
	FILE *f;
	f = fopen("bdvente.bd", "rb");
	if (f==NULL)
		return;

	//Création d'une zone mémoire pour la lecture depuis le fichier
	struct infovente *vente = (struct infovente *) malloc(sizeof(struct infovente));

	//Ajout des éléments extraits du fichier source
	fread(vente, sizeof(struct infovente), 1, f);
	while(!feof(f)){
		addItem(vente->nomvendeur, vente->noproduit, vente->nombrevendu);
		fread(vente, sizeof(struct infovente), 1, f);
		}

	//Libération de la zone mémoire pour la lecture depuis le fichier
	free(vente);
 	}

//#######################################
//#
//# Enregistre le contenu de la liste chaînée dans un fichier binaire
//#
void saveItems(){

	struct record * optr = head;			//dummy element
	struct record * ptr = head->suivant;	//first element
	FILE *f;

	//Ouverture du fichier en mode "wb" : [w]rite [b]inary
	f = fopen("bdvente.bd", "wb");
	if (f==NULL)
		error(2, "saveItems: Erreur lors de l'ouverture du fichier pour écriture en mode binaire.");

	while (ptr!=NULL){

		//Écriture des données
		fwrite(&ptr->vente, sizeof(struct infovente), 1, f);

		//Déplacement des 2 pointeurs
		optr = ptr;
		ptr = ptr->suivant;
		}

	//Fermeture du fichier
	fclose(f);
	}

//#######################################
//#
//# fonction utilisée par les threads de transactions
//#
void* readTrans(char* nomFichier){
	FILE *f;
	char buffer[100];
	char *tok, *sp;

	//Ouverture du fichier en mode "r" (equiv. "rt") : [r]ead [t]ext
	f = fopen(nomFichier, "r");
	if (f==NULL)
		error(2, "readTrans: Erreur lors de l'ouverture du fichier.");

	//Lecture (tentative) d'une ligne de texte
	fgets(buffer, 100, f);

	//Pour chacune des lignes lues
	while(!feof(f)){

		//Extraction du type de transaction
		tok = strtok_r(buffer, " ", &sp);

		//Branchement selon le type de transaction
		switch(tok[0]){
			case 'A':
			case 'a':{
				//Extraction des paramètres
				char *nv = strtok_r(NULL, " ", &sp);
				char *np = strtok_r(NULL, " ", &sp);
				int nbv = atoi(strtok_r(NULL, " ", &sp));
				//Appel de la fonction associée
				addItem(nv, np, nbv);
				break;
				}
			case 'E':
			case 'e':{
				//Extraction du paramètre
				int ns = atoi(strtok_r(NULL, " ", &sp));
				//Appel de la fonction associée
				removeItem(ns);
				break;
				}
			case 'M':
			case 'm':{
				//Extraction des paramètres
				int ns = atoi(strtok_r(NULL, " ", &sp));
				char *nv = strtok_r(NULL, " ", &sp);
				char *np = strtok_r(NULL, " ", &sp);
				int nbv = atoi(strtok_r(NULL, " ", &sp));
				//Appel de la fonction associée
				modifyItem(ns, nv, np, nbv);
				break;
				}
			case 'L':
			case 'l':{
				//Extraction des paramètres
				int nstart = atoi(strtok_r(NULL, "-", &sp));
				int nend = atoi(strtok_r(NULL, " ", &sp));
				//Appel de la fonction associée
				listItems(nstart, nend);
				break;
				}
			case 'S':
			case 's':{
				//Appel de la fonction associée
				saveItems();
				break;
				}
			}

		//Lecture (tentative) de la prochaine ligne de texte
		fgets(buffer, 100, f);
		}

	//Fermeture du fichier
	fclose(f);

	//Retour
	return NULL;
	}


